/* Name: main.c
 * Project: HID-LightCtrl
 * Author: SFR frydaysoft.de
 * Creation Date: 2013-03-19
 * Tabsize: 4
 * Note: basics used from OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt)
 * This Revision: $Id: main.c 788 2010-05-30 20:54:41Z cs $
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/boot.h>
#include <string.h>
#include <util/delay.h>

static void startBootloader() __attribute__((__noreturn__));

#include "nrf/nrf_types.h"
#include "nrf/nrf_drv.h"

#include "hwconfig.h"
#include "usbdrv.c"

/* ------------------------------------------------------------------------ */

#ifndef ulong
#   define ulong    unsigned long
#endif
#ifndef uint
#   define uint     unsigned int
#endif

typedef struct
{
  uchar id;
  uchar len;
} T_sReportReq;
// *********************************************************************************
static uchar          versionBuffer[7] = { 2,'V','1','.','0','0',0}; 
static uchar          cmdBuffer[7];
static uchar          bLedTest;
static T_sReportReq   sReportReq;
static uchar          NrfCmdBuf[4];

#if 0
BYTE Nrf_TxMasterAddr[] = {'l', 'u', 's', 'b', '0'};
BYTE Nrf_RxMasterAddr[] = {'l', 'u', 's', 'b', '1'};
#else
//-------------------------------------------------------
BYTE Nrf_TxMasterAddr[] = {'T', 'x', '_', '0', '0'}; // nrf address tx data
BYTE Nrf_RxMasterAddr[] = {'R', 'x', '_', '0', '0'}; // nrf address rx data
#endif

#define ID_REP_1  1
#define LEN_REP_1 6

#define ID_REP_2  2
#define LEN_REP_2 6

// *********************************************************************************
PROGMEM char usbHidReportDescriptor[33] = {
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)

    0x85, ID_REP_1,                //   REPORT_ID (1=simple cmd)
    0x95, LEN_REP_1,               //   REPORT_COUNT (6)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)

    0x85, ID_REP_2,                //   REPORT_ID (2=GetVersion)
    0x95, LEN_REP_2,               //   REPORT_COUNT (6)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};

/* compatibility with ATMega88 and other new devices: */
#ifndef TCCR0
#define TCCR0   TCCR0B
#endif
#ifndef GICR
#define GICR    MCUCR
#endif

// ************************************************************************
// ***** startBootloader() ************************************************
// ************************************************************************
static void startBootloader()
{
    DBG1(0x01, 0, 0);
    cli();
    /* activate pull-ups except on USB lines */
    USB_CFG_IOPORT   = (uchar)~((1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT));
    /* all pins input except USB (-> USB reset) */
#ifdef USB_CFG_PULLUP_IOPORT    /* use usbDeviceConnect()/usbDeviceDisconnect() if available */
    USBDDR    = 0;    /* we do RESET by deactivating pullup */
    usbDeviceDisconnect();
#else
    USBDDR    = (1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT);
#endif
    wdt_enable(WDTO_1S);
    while(1) {;}
}

// ************************************************************************
// ***** SendLedCommand() *************************************************
// ************************************************************************
void SendLedCommand(void)
{
  if (NRF24L_IS_IRQ())
  { // ***** irq from last TX
    NrfDrvClearInt(NRF_ALL_IRQ);
    NrfDrvWriteFifoData(NrfCmdBuf, 4);
    if (!NRF24L_IS_IRQ())
    { // ***** irq line released by cmd
      Led1Tog();
    }
  }
  else
  { // ***** e.g. first cmd after power down or reset
    NrfDrvWriteFifoData(NrfCmdBuf, 4);
  }
  NrfHwTransmitPulse();
}

// ************************************************************************
// ***** usbFunctionSetup() ***********************************************
// ************************************************************************
uchar usbFunctionSetup(uchar data[8])
{
    uchar  ret = 0;

    // eg: 21,09,01,03,00,00,07 for HidD_SetFeature("01 ......")
    // data[0]   = rq->bmRequestType
    // data[1]   = rq->bRequest
    // data[2/3] = rq->wValue
    // data[4/5] = rq->wIndex
    // data[6/7] = rq->wLength

    // ***** rq->bRequest
    switch(data[1]) 
    {
      case USBRQ_HID_SET_REPORT:  // 0x09
        sReportReq.id  = data[2]; // report ID
        sReportReq.len = data[6]; // report length
        ret = USB_NO_MSG;
        break;
      case USBRQ_HID_GET_REPORT:  // 0x01
        switch(data[2])           // report ID
        {
          case ID_REP_1:
            usbMsgPtr = cmdBuffer;
            ret = (LEN_REP_1+1);
            break;
          case ID_REP_2:
            usbMsgPtr = versionBuffer;
            ret = (LEN_REP_2+1);
            break;
          default:
            break;
        }
      default:
        break;
    }
    return ret;
}

// ************************************************************************
// ***** usbFunctionWrite() ***********************************************
// ************************************************************************
uchar usbFunctionWrite(uchar *data, uchar len)
{
  if (sReportReq.len)
  {
    if (sReportReq.len == len)
    {
      switch(sReportReq.id)
      { 
        // ***** simple cmd, max. 5 bytes left
        case ID_REP_1:
          memcpy(&cmdBuffer[0], &data[0], LEN_REP_1);
          switch(data[1])
          { // ***** set lights
            case 0:
              NrfCmdBuf[2] = data[2];   // lights[0..7] for old message ">3Lxx"
              NrfCmdBuf[3] = data[3];   // lights[8..F] for new message ">5Lxxyy"
              SendLedCommand();
              break;
            // ***** start/stop LED test
            case 1:
              bLedTest = data[2];
              break;
            default:
              break;
          }
          break;
        // ***** get version has no write access
        case ID_REP_2:
        default:
          break;
      }
    }
  }
  sReportReq.id  = 0;
  sReportReq.len = 0;
  return 0x80; /* != 0 if last block received */
}


#define USE_T1
// ************************************************************************
// ***** TimerInit() ******************************************************
// ************************************************************************
void TimerInit(void)
{ 
#ifdef USE_T1
  // ***** stop Timer1
  TCCR1A = 0x00;
  TCCR1B = 0x00;   
  // ***** set Output Compare Register
  OCR1A  = 18750;  // 1200000 / 64 * 0,1 = 18750
  TIFR   = (1<<OCF1A); 
  TCCR1B = 0x0B;   // clear Timer1 on CTC, prescaler 64
#else
#endif
}

// ************************************************************************
// ***** TimerPoll() ******************************************************
// ************************************************************************
uchar TimerPoll()
{
#ifdef USE_T1
  if (TIFR & (1<<OCF1A))
  {
    TIFR  |= (1<<OCF1A); 
	  return 1;
  }
  return 0;
#else
  return 1;
#endif
}

// ************************************************************************
// ***** initForUsbConnectivity() *****************************************
// ************************************************************************
static void initForUsbConnectivity(void)
{
    uchar   i = 0;

#if F_CPU == 12800000
    TCCR0 = 3;          /* 1/64 prescaler */
#endif
    usbInit();
    /* enforce USB re-enumerate: */
    usbDeviceDisconnect();  /* do this while interrupts are disabled */
    do{             /* fake USB disconnect for > 250 ms */
        wdt_reset();
        _delay_ms(1);
    }while(--i);
    usbDeviceConnect();
    sei();
}

// ************************************************************************
// ***** SpiInit() ********************************************************
// ************************************************************************
void SpiInit(void)
{
  volatile char IOReg;
  // Note: 
  // PB2 (/SS) will be used as output (CE of NRF chip) to avoid SPI slave select problem
  // ***** enable SPI in Master Mode with SCK = CK/4 (3MHz)
  SPCR    = (1<<SPE)|(1<<MSTR);
  SPSR    = (1 << SPI2X);
  // ***** set PB3(MOSI), PB5(SCK) as output 
  DDRB    = (1<<PB3)|(1<<PB5);
  // ***** clear SPIF bit in SPSR
  IOReg   = SPSR;                 	
  IOReg   = SPDR;
  NRF24L_SET_CE_LOW();
  NRF24L_SET_CS_HIGH();
}

// ************************************************************************
// ***** hardwareInit() ***************************************************
// ************************************************************************
void hardwareInit(void)
{
    uchar   i;

    usbDeviceDisconnect();  // enforce re-enumeration, do this while interrupts are disabled!
    i = 0;
    while(--i)
    { // ***** fake USB disconnect for > 250 ms
      wdt_reset();
      _delay_ms(1);
    }
    usbDeviceConnect();
}

// ************************************************************************
// ***** main() ***********************************************************
// ************************************************************************
int __attribute__((noreturn)) main(void)
{
    uchar   steps=0;

    wdt_enable(WDTO_1S); 
    odDebugInit();
    DBG1(0x00, 0, 0);
    // ***** initialize hardware
    HwLowInit();
    InitLeds();
    SpiInit();          // call at first SpiInit() and then configure /SS as output
    NRF24L_INIT_PINS();
    NrfHwInit(NRF_RF_PWR_0dBm, NRF_2MBPS, NRF_LNA_HCURR);
    // 10ms nrf24l boot time
    hardwareInit();
    sReportReq.len = 0;

    // ***** NRF on Channel 2 
    NrfDrvSetTxMaster(2, Nrf_TxMasterAddr, NRF_AW_5BYTES); 
    usbInit();

    sei();
    DBG1(0x01, 0, 0);
    TimerInit();

    bLedTest = 0;
    steps    = 0; 
    NrfCmdBuf[0] = 3;   // byte count
    NrfCmdBuf[1] = 'L'; // cmd
    NrfCmdBuf[2] = 0;   // hi lights
    NrfCmdBuf[3] = 0;   // lo lights
    NrfDrvWriteFifoData(NrfCmdBuf, 4);

    // ***** main event loop 
    do
    { 
      wdt_reset();
      usbPoll();
      if (TimerPoll())
      {
        if (++steps >= 10)
        {
          if (bLedTest)
          {
            if (!NrfCmdBuf[2])
            {
              NrfCmdBuf[3] <<= 1;
              if (!NrfCmdBuf[3])
              {
                NrfCmdBuf[2] = 1;
              }
            }
            else
            {
              NrfCmdBuf[2] <<= 1;
              if (!NrfCmdBuf[2])
              {
                NrfCmdBuf[3] = 1;
              }
            }
            SendLedCommand();
          }
          Led2Tog();
          steps = 0;
        }
      } 
    }while(AppCondition());

    startBootloader();
}

