/* 
 * Project     : WinAmp USB LightCtrl via USB
 * Author      : frydaysoft.de (based on an example from Stanimir Bonev.)
 * File name   : nrv_drv.c
 * Description : Nordic nRF24L01 driver functions
 *  
 * Copyright: (c) 2013 frydaysoft.de 
 * License  : Proprietary, free under certain conditions. See Documentation.
 *
 * Creation Date: 2013-04-27
 */

#include "nrf_types.h"
#include "nrf_drv.h"


/*************************************************************************
 * Function Name: NrfDrvMode()
 * Parameters: Mode Type (NRF_RX_MODE, NRF_TX_MODE)
 *
 * Description: Sets the Nrf24L01 Rx Tx Mode
 *
 *************************************************************************/
void NrfDrvMode(NrfModeType eMode)
{
  BYTE Config;

  Config = NrfHwReadRegByte(NRF_REG_CONFIG);  // Read Nrf Config Register
  if (NRF_RX_MODE == eMode)
  { // ***** Rx Mode
    Config |= NRF_MSK_CONFIG__PRIM_RX;        // Set mode Bit
  }
  else
  { // ***** Tx Mode
    Config &= ~NRF_MSK_CONFIG__PRIM_RX;       // Clear Mode Bit
  }
  NrfHwWriteRegByte(NRF_REG_CONFIG, Config);  // Write Nrf Config Register	
}

/*************************************************************************
 * Function Name: NrfDrvPower()
 * Parameters: Power Mode (NRF_POWER_DOWN, NRF_POWER_UP)
 *
 * Description: Sets the Nrf24L01 Power Mode
 *
 *************************************************************************/
void NrfDrvPower(NrfPowerType ePower)
{
  BYTE Config;

  Config = NrfHwReadRegByte(NRF_REG_CONFIG);    // Read Nrf Config Register
  if (NRF_POWER_UP == ePower)
  { // ***** Power Up
    Config |= NRF_MSK_CONFIG__PWR_UP;           // Set Power Up Bit
  }
  else
  { // ***** Power Down
    Config &= ~NRF_MSK_CONFIG__PWR_UP;          // Clear Power Up Bit
  }
  NrfHwWriteRegByte(NRF_REG_CONFIG, Config);    // Write Nrf Config Register	
}	

/*************************************************************************
 * Function Name: NrfDrvCrcType()
 * Parameters: CRC width (NRF_CRC16, NRF_CRC8)
 *
 * Description: Sets the Nrf24L01 Power Mode
 *
 *************************************************************************/
void NrfDrvCrcType(NrfCrcType eCrc)
{
  BYTE Config;

  Config = NrfHwReadRegByte(NRF_REG_CONFIG);    // Read Nrf Config Register
  if (NRF_CRC16 == eCrc)
  { // ***** 2 Bytes CRC
    Config |= NRF_MSK_CONFIG__CRCO;             // Set CRCO Bit
  }
  else
  { // ***** 1 Byte CRC
    Config &= ~NRF_MSK_CONFIG__CRCO;            // Clear CRCO Bit
  }
  NrfHwWriteRegByte(NRF_REG_CONFIG, Config);    // Write Nrf Config Register	
}

/*************************************************************************
 * Function Name: NrfDrvCrcEn()
 * Parameters: crc enable/disable (NRF_CRC_ENABLE, NRF_CRC_DISABLE)
 *
 * Description: Enables or Disables Nrf CRC
 *
 *************************************************************************/
void NrfDrvCrcEn(NrfCrcEnType eEnable)
{
  BYTE Config;

  Config = NrfHwReadRegByte(NRF_REG_CONFIG);      // Read Nrf Config Register
  if (NRF_CRC_ENABLE == eEnable)
  { // ***** Enable CRC
    Config |= NRF_MSK_CONFIG__EN_CRC;             // Set CRC Enable Bit
  }
  else
  { // ***** Disable CRC
    Config &= ~NRF_MSK_CONFIG__EN_CRC;            // Clear CRC Enable Bit
  }
  NrfHwWriteRegByte(NRF_REG_CONFIG, Config);      // Write Nrf Config Register	
}

/*************************************************************************
 * Function Name: NrfDrvMaskInt()
 * Parameters: Interrupt Mask (NRF_TX_DS_IRQ, RX_DR, MAX_RT)
 *
 * Description: Mask Nrf Interrupt
 *
 *************************************************************************/
void NrfDrvMaskInt(BYTE Interrupt)
{
  BYTE Config;

  Config = NrfHwReadRegByte(NRF_REG_CONFIG);         // Read Config Register
  Config |= (Interrupt & (NRF_ALL_IRQ));             // Set mask Bits
  NrfHwWriteRegByte(NRF_REG_CONFIG, Config);         // Write Nrf Config Register
}

/*************************************************************************
 * Function Name: NrfDrvUnmaskInt()
 * Parameters: Interrupt (NRF_TX_DS, NRF_RX_DR, NRF_MAX_RT)
 *
 * Description: UnMask Nrf Interrupt
 *
 *************************************************************************/
void NrfDrvUnmaskInt(BYTE Interrupt)
{
  BYTE Config;

  Config = NrfHwReadRegByte(NRF_REG_CONFIG);          // Read Config Register
  Config &= ~(Interrupt & (NRF_ALL_IRQ));             // Clear mask Bits
  NrfHwWriteRegByte(NRF_REG_CONFIG, Config);          // Write Nrf Config Register
}

/*************************************************************************
 * Function Name: NrfDrvEnAutoAck()
 * Parameters: Pipe Mask to be Enabled (NRF_PIPE_0M..NRF_PIPE_5M)
 *
 * Description: Enable Pipe Auto Ack
 *
 *************************************************************************/
void NrfDrvEnAutoAck(BYTE Pipe)
{
  BYTE AutoAck;

  AutoAck = NrfHwReadRegByte(NRF_REG_EN_AA);  // Read EN_AA Register
  AutoAck |= (Pipe & (NRF_ALL_PIPES_M));      // Set Enable Bits
  NrfHwWriteRegByte(NRF_REG_EN_AA, AutoAck);  // Write Nrf EN_AA Register
}

/*************************************************************************
 * Function Name: NrfDrvDisAutoAck()
 * Parameters: Pipe to be Disabled (NRF_PIPE_0M..NRF_PIPE_5M)
 *
 * Description: Disables Pipe Auto Ack
 *
 *************************************************************************/
void NrfDrvDisAutoAck(BYTE Pipe)
{
  BYTE AutoAck;

  AutoAck = NrfHwReadRegByte(NRF_REG_EN_AA); // Read EN_AA Register
  AutoAck &= ~(Pipe & (NRF_ALL_PIPES_M));    // Clear Enable Bits
  NrfHwWriteRegByte(NRF_REG_EN_AA, AutoAck); // Write Nrf EN_AA Register
}

/*************************************************************************
 * Function Name: NrfDrvEnRxAddr()
 * Parameters: Pipe to be Enabled (NRF_PIPE_0M..NRF_PIPE_5M)
 *
 * Description: Enable RX Address
 *
 *************************************************************************/
void NrfDrvEnRxAddr(BYTE Pipe)
{
  BYTE RxAddr;

  RxAddr = NrfHwReadRegByte(NRF_REG_EN_RXADDR); // Read EN_RXADDR Register
  RxAddr |= (Pipe & (NRF_ALL_PIPES_M));         // Set Enable Bits
  NrfHwWriteRegByte(NRF_REG_EN_RXADDR, RxAddr); // Write Nrf EN_AA Register
}

/*************************************************************************
 * Function Name: NrfDrvDisRxAddr()
 * Parameters: Pipe to be Enabled (NRF_PIPE_0M..NRF_PIPE_5M)
 *
 * Description: Enable RX Address
 *
 *************************************************************************/
void NrfDrvDisRxAddr(BYTE Pipe)
{
  BYTE RxAddr;

  RxAddr = NrfHwReadRegByte(NRF_REG_EN_RXADDR); // Read EN_RXADDR Register
  RxAddr &= ~(Pipe & (NRF_ALL_PIPES_M));        // Set Enable Bits
  NrfHwWriteRegByte(NRF_REG_EN_RXADDR, RxAddr); // Write Nrf EN_AA Register
}

/*************************************************************************
 * Function Name: NrfDrvAddrWidth()
 * Parameters: Address width (NRF_AW_3BYTES..NRF_AW_5BYTES)
 *
 * Description: Sets Nrf Address Width
 *
 *************************************************************************/
void NrfDrvAddrWidth(NrfAddrWidthType eAddrWidth)
{
  NrfHwWriteRegByte(NRF_REG_SETUP_AW, (BYTE)eAddrWidth & NRF_AW_5BYTES); 
}

/*************************************************************************
 * Function Name: NrfDrvAutoRetDelay()
 * Parameters: Auto Retransmit Delay (NRF_ARD_250us..NRF_ARD_4000us)
 *
 * Description: Sets Nrf Auto Retransmit Delay
 *
 *************************************************************************/
void NrfDrvAutoRetDelay(NrfAutoDelayType eDelay)
{
  BYTE SetupRetr;

  SetupRetr = NrfHwReadRegByte(NRF_REG_SETUP_RETR);
  SetupRetr &= ~(NRF_ARD_4000us);
  SetupRetr |= ( eDelay & NRF_ARD_4000us);
  NrfHwWriteRegByte(NRF_REG_SETUP_RETR, SetupRetr);
}

/*************************************************************************
 * Function Name: NrfDrvAutoRetCount()
 * Parameters: Byte ( 0 .. 15 ), How many times the frame should be
 *             retransmitted
 *
 * Description: Sets Auto Retransmit Count
 *
 *************************************************************************/
void NrfDrvAutoRetCount(BYTE Count)
{
  BYTE SetupRetr;

  SetupRetr = NrfHwReadRegByte(NRF_REG_SETUP_RETR);
  SetupRetr &= ~NRF_MSK_SETUP_RETR__ARC;
  SetupRetr |= (Count & NRF_MSK_SETUP_RETR__ARC);
  NrfHwWriteRegByte(NRF_REG_SETUP_RETR, SetupRetr);
}

/*************************************************************************
 * Function Name: NrfDrvLNA()
 * Parameters: LNA Current (NRF_LNA_LCURR, NRF_LNA_HCURR)
 *
 * Description: Sets Auto LNA Current
 *
 *************************************************************************/
void NrfDrvLNA(NrfLnaType eCurrent)
{
  BYTE RfSetup;

  RfSetup = NrfHwReadRegByte(NRF_REG_RF_SETUP);
  if (NRF_LNA_HCURR == eCurrent)
  {
    RfSetup |= NRF_MSK_RF_SETUP__RF_LNA;
  }
  else
  {
    RfSetup &= ~NRF_MSK_RF_SETUP__RF_LNA;
  }
  NrfHwWriteRegByte(NRF_REG_RF_SETUP, RfSetup);
}

/*************************************************************************
 * Function Name: NrfDrvOutPower()
 * Parameters: Output Power(NRF_RF_PWR_0dBm..NRF_RF_PWR_18dBm)
 *
 * Description: Sets Output Power
 *
 *************************************************************************/
void NrfDrvOutPower(NrfOutPowerType ePower)
{
  BYTE RfSetup;

  RfSetup = NrfHwReadRegByte(NRF_REG_RF_SETUP);
  RfSetup &= ~NRF_RF_PWR_0dBm;
  RfSetup |= (ePower & NRF_RF_PWR_0dBm);
  NrfHwWriteRegByte(NRF_REG_RF_SETUP, RfSetup);
}

/*************************************************************************
 * Function Name: NrfDrvDataRate()
 * Parameters: Data Rate (NRF_1MBPS, NRF_2MBPS)
 *
 * Description: Sets Nrf Air Data Rate
 *
 *************************************************************************/
void NrfDrvDataRate(NrfDataRateType eRate)
{
  BYTE RfSetup;

  RfSetup = NrfHwReadRegByte(NRF_REG_RF_SETUP);
  if (NRF_2MBPS == eRate)
  {
    RfSetup |= NRF_MSK_RF_SETUP__RF_DR;
  }
  else
  {
    RfSetup &= ~NRF_MSK_RF_SETUP__RF_DR;
  }
  NrfHwWriteRegByte(NRF_REG_RF_SETUP, RfSetup);
}

/*************************************************************************
 * Function Name: NrfDrvSetRxAddr()
 * Parameters: Pipe, pointer to Addr array, Address Size
 * Return: 1 on success, 0 on error
 *
 * Description: Sets Pipe Rx Address
 *
 *************************************************************************/
BOOL NrfDrvSetRxAddr(NrfPipeType ePipe, BYTE *pAddr, BYTE Size)
{
  if (Size > NRF_MAX_ADDR_LEN) return 0;
  if (ePipe > NRF_PIPE5)       return 0;
  if (pAddr && Size)
  {
    NrfHwWriteReg(NRF_REG_RX_ADDR_P0+ePipe, pAddr, Size);
  }
  return 0;
}

/*************************************************************************
 * Function Name: NrfDrvSetTxAddr()
 * Parameters: pointer to Addr array, Address Size
 * Return: 1 on success, 0 on error
 *
 * Description: Sets Tx Address
 *
 *************************************************************************/
BOOL NrfDrvSetTxAddr(BYTE *pAddr, BYTE Size)
{
  if (Size > NRF_MAX_ADDR_LEN) return 0;
  if (pAddr && Size)
  {
    NrfHwWriteReg(NRF_REG_TX_ADDR, pAddr, Size);
    return 1;
  }
  return 0;
}

/*************************************************************************
 * Function Name: NrfDrvRxPayWidth()
 * Parameters: Pipe, Payload Width
 * Return: 1 on success, 0 on error
 *
 * Description: Sets Pipe Payload Width
 *
 *************************************************************************/
BOOL NrfDrvRxPayWidth(NrfPipeType ePipe, BYTE PayWidth)
{
  if(ePipe > NRF_PIPE5)            return 0;
  if(PayWidth > NRF_RX_FIFO_SIZE)  return 0;
  NrfHwWriteRegByte(NRF_REG_RX_PW_P0+ePipe, PayWidth);
  return 1;
}

/*************************************************************************
 * Function Name: NrfDrvEnDynPay()
 * Parameters: Pipe to Enable (NRF_PIPE_0M..NRF_PIPE_5M)
 *
 * Description: Enables Pipe Dynamic Payload length
 *
 *************************************************************************/
void NrfDrvEnDynPay(BYTE Pipe)
{
  BYTE DynPay;

  DynPay = NrfHwReadActRegByte(NRF_REG_DYNPD);  // Read DYNPD Register
  DynPay |= (Pipe & (NRF_ALL_PIPES_M));         // Set Enable Bits
  NrfHwWriteActRegByte(NRF_REG_DYNPD, DynPay);  // Write Nrf DYNPD Register
}

/*************************************************************************
 * Function Name: NrfDrvDisDynPay()
 * Parameters: Pipe to Disable (NRF_PIPE_0M..NRF_PIPE_5M)
 *
 * Description: Disable Pipe Dynamic Payload length
 *
 *************************************************************************/
void NrfDrvDisDynPay(BYTE Pipe)
{
  BYTE Dynpd;

  Dynpd = NrfHwReadActRegByte(NRF_REG_DYNPD); // Read DYNPD Register
  Dynpd &= ~(Pipe & (NRF_ALL_PIPES_M));       // Set Enable Bits
  NrfHwWriteActRegByte(NRF_REG_DYNPD, Dynpd); // Write Nrf DYNPD Register
}

/*************************************************************************
 * Function Name: NrfDrvDynPay()
 * Parameters: Enable Disable (DPL_ENABLE, DPL_DISABLE)
 *
 * Description: Enables/Disables Dynamic Payload length
 *
 *************************************************************************/
void NrfDrvDynPay(NrfDynPayType eDynPay)
{
  BYTE Feature;

  Feature = NrfHwReadActRegByte(NRF_REG_FEATURE);
  if (NRF_DPL_ENABLE == eDynPay)
  {
    Feature |= NRF_MSK_FEATURE__EN_DPL;
  }
  else
  {
    Feature &= ~NRF_MSK_FEATURE__EN_DPL;
  }
  NrfHwWriteActRegByte(NRF_REG_FEATURE, Feature);
}

/*************************************************************************
 * Function Name: NrfDrvDynAck()
 * Parameters: Enable Disable (DYN_ACK_ENABLE, DYN_ACK_DISABLE)
 *
 * Description: Enables/Disables Dynamic Payload length
 *
 *************************************************************************/
void NrfDrvDynAck(NrfDynAckType eDynAck)
{
  BYTE Feature;

  Feature = NrfHwReadActRegByte(NRF_REG_FEATURE);
  if (NRF_DYN_ACK_ENABLE == eDynAck)
  {
    Feature |= NRF_MSK_FEATURE__EN_DYN_ACK;
  }
  else
  {
    Feature &= ~NRF_MSK_FEATURE__EN_DYN_ACK;
  }
  NrfHwWriteActRegByte(NRF_REG_FEATURE, Feature);
}

/*************************************************************************
 * Function Name: NrfDrvAckPay()
 * Parameters: Enable Disable (DYN_ACK_ENABLE, DYN_ACK_DISABLE)
 *
 * Description: Enables/Disables Dynamic Payload length
 *
 *************************************************************************/
void NrfDrvAckPay(NrfAckPayType eAckPay)
{
  BYTE Feature;

  Feature = NrfHwReadActRegByte(NRF_REG_FEATURE);
  if (NRF_ACK_PAY_ENABLE == eAckPay)
  {
    Feature |= NRF_MSK_FEATURE__EN_ACK_PAY;
  }
  else
  {
    Feature &= ~NRF_MSK_FEATURE__EN_ACK_PAY;
  }
  NrfHwWriteActRegByte(NRF_REG_FEATURE, Feature);
}

/*************************************************************************
 * Function Name: NrfDrvGetPipeAvb()
 * Parameters: none
 * Return: pipe number
 *
 * Description: Gets the Pipe number for the payload available for
 *              reading from RX_FIFO
 *
 *************************************************************************/
BYTE NrfDrvGetPipeAvb(void)
{
  BYTE Status;

  Status = NrfHwStatus();
  return (NRF_GET_RX_NO(Status));
}

/*************************************************************************
 * Function Name: NrfDrvGetState()
 * Parameters: none
 * Return: state of the NRF
 *
 * Description: Returns the State of the Nrf24L01
 *
 *************************************************************************/
NrfStateType NrfDrvGetState(void)
{
  BYTE Reg;

  Reg = NrfHwReadRegByte(NRF_REG_CONFIG);
  if (!(Reg & NRF_MSK_CONFIG__PWR_UP))
  { 
    return(NRF_ST_PWR_DOWN);
  }
  if (Reg & NRF_MSK_CONFIG__PRIM_RX)
  {
    if (NRF24L_GET_CE()) 
    {
      return(NRF_ST_RX);
    }
    return(NRF_ST_STAND_I);
  }

  Reg = NrfHwReadRegByte(NRF_REG_FIFO_STATUS);
  if (!(Reg & NRF_MSK_FIFO_STATUS__TX_FIFO_EMPTY)) 
  {
    return(NRF_ST_TX);
  }

  if (NRF24L_GET_CE()) 
  {
    return(NRF_ST_STAND_II);
  }
  return(NRF_ST_STAND_I);
}


/*************************************************************************
 * Function Name: NrfDrvGetIntID()
 * Parameters: none
 * Return: NrfIrqState
 *
 * Description: Returns the State of the Nrf24L01 IRQs
 *
 *************************************************************************/
BYTE NrfDrvGetIntID(void)
{
  BYTE Status, Mask;

  Status = NrfHwReadRegByte(NRF_REG_STATUS);
  Mask   = NrfHwReadRegByte(NRF_REG_CONFIG);
  Mask  |= ~(NRF_ALL_IRQ);
  return((BYTE)(Status & ~Mask));
}

/*************************************************************************
 * Function Name: NrfDrvSetPowerDown()
 * Parameters: none
 *
 * Description: Set Nrf24L01 into power down mode
 *
 *************************************************************************/
void NrfDrvSetPowerDown(void)
{
  NrfHwFlushRx();
  NrfHwFlushTx();
  NrfDrvPower(NRF_POWER_DOWN);
  NrfDrvClearInt(NRF_ALL_IRQ);
  NRF24L_SET_CE_LOW();
}

/*************************************************************************
 * Function Name: NrfDrvSetRxSlave()
 * Parameters: Communication channel and Communication Address
 *
 * Description: A default configuration for simple RX with Pipe 0
 *
 *************************************************************************/
void NrfDrvSetRxSlave(BYTE channel, BYTE *pAddr, NrfAddrWidthType eAddrWidth)
{
  // ***** Disable interrupt
  NrfDrvMaskInt(NRF_ALL_IRQ);
#if 0
  NrfDrvDisAutoAck(NRF_PIPE_1_TO_5_M);
  NrfDrvEnAutoAck(NRF_PIPE_0M);
#else
  NrfDrvDisAutoAck(NRF_ALL_PIPES_M);  // keep disabled to be compatible with nRF2401
#endif
  NrfDrvEnRxAddr(NRF_PIPE_0M);
  // ***** set Communication Address
  NrfDrvAddrWidth(eAddrWidth);
  NrfDrvSetRxAddr(NRF_PIPE0, pAddr, 2 + eAddrWidth);
  NrfHwFlushRx();
  NrfDrvPower(NRF_POWER_UP); 
  NrfDrvMode(NRF_RX_MODE);
  NrfDrvClearInt(NRF_ALL_IRQ);
  NrfDrvUnmaskInt(NRF_MAX_RT_IRQ | NRF_RX_DR_IRQ);
  NrfDrvSetChannel(channel);
  NrfDrvEnDynPay(NRF_PIPE_0M);
  NrfDrvDynPay(NRF_DPL_ENABLE);
  NRF24L_SET_CE_HIGH();
}

/*************************************************************************
 * Function Name: NrfDrvSetTxMaster()
 * Parameters: Communication channel and Communication Address
 *
 * Description: A default configuration for simple TX with Pipe 0
 *
 *************************************************************************/
void NrfDrvSetTxMaster(BYTE channel, BYTE *pAddr, NrfAddrWidthType eAddrWidth)
{
  NRF24L_SET_CE_LOW();
  // ***** Disable interrupt
  NrfDrvMaskInt(NRF_ALL_IRQ);
  NrfDrvDisAutoAck(NRF_ALL_PIPES_M);  // keep disabled to be compatible with nRF2401
  // ***** set Communication Address
  NrfDrvAddrWidth(eAddrWidth);
  NrfDrvSetTxAddr(pAddr, 2 + eAddrWidth);
  NrfHwFlushTx();
  NrfDrvPower(NRF_POWER_UP); 
  NrfDrvMode(NRF_TX_MODE);
  NrfDrvClearInt(NRF_ALL_IRQ);
  NrfDrvUnmaskInt(NRF_MAX_RT_IRQ | NRF_TX_DS_IRQ);
  //Auto retransmit 3 times, 500us between retransmissions
  NrfDrvAutoRetCount(3);
  NrfDrvAutoRetDelay(NRF_ARD_500us);
  NrfDrvSetChannel(channel);
  NrfDrvEnDynPay(NRF_PIPE_0M);
  NrfDrvDynPay(NRF_DPL_ENABLE);
  NRF24L_SET_CE_HIGH();
}




