/* 
 * Project     : WinAmp USB LightCtrl via USB
 * Author      : frydaysoft.de (based on an example from Stanimir Bonev.)
 * File name   : nrv_hw.c
 * Description : Nordic nRF24L01 hw functions
 *  
 * Copyright: (c) 2013 frydaysoft.de 
 * License  : Proprietary, free under certain conditions. See Documentation.
 *
 * Creation Date: 2013-04-27
 */

#include "nrf_types.h"
#include "nrf_drv.h"

BYTE _nrfLastResult;

/*************************************************************************
 * Function Name: _NrfHwSendActComm
 * Parameters: None
 *
 * Return: None
 *
 * Description: internal utility function for: NrfHwWriteActReg 
 *              and NrfHwReadActReg
 *
 *************************************************************************/
void _NrfHwSendActComm(void)
{
	NRF24L_SET_CS_LOW();
	NrfHwSpiSend(NRF_CMD_ACTIVE);
	NrfHwSpiSend(NRF_CMD_ACTIVE_DATA);
	NRF24L_SET_CS_HIGH();
}

/*************************************************************************
 * Function Name: NrfHwInit
 * Parameters: None
 *
 * Return: None
 *
 * Description: Returns the State of the Nrf24L01 IRQs
 *
 *************************************************************************/
void NrfHwInit(NrfOutPowerType ePower, NrfDataRateType eRate, NrfLnaType eCurrent)
{
  _nrfLastResult = 0;
  //Power Down Mode
  NrfDrvPower(NRF_POWER_DOWN);
  //Disable interrupt
  NrfDrvMaskInt(NRF_ALL_IRQ);
  //Enable CRC
  NrfDrvCrcType(NRF_CRC16);
  NrfDrvCrcEn(NRF_CRC_ENABLE);
  //0 dBm output Power, 2Mbps, LNA High Current
  NrfDrvOutPower(ePower);
  NrfDrvDataRate(eRate);
  NrfDrvLNA(eCurrent);

  NrfDrvDisDynPay(NRF_ALL_PIPES_M);
  NrfDrvDynPay(NRF_DPL_DISABLE);

  //Clear Interrupts
  NrfDrvClearInt(NRF_ALL_IRQ);
}

/*************************************************************************
 * Function Name: NrfHwStatus
 * Parameters: None
 *
 * Return: Value of STATUS register
 *
 * Description: Returns the State of the Nrf24L01
 *
 *************************************************************************/
BYTE NrfHwLastResult(void)
{
  return _nrfLastResult;
}

/*************************************************************************
 * Function Name: NrfHwStatus
 * Parameters: None
 *
 * Return: Value of STATUS register
 *
 * Description: Returns the State of the Nrf24L01
 *
 *************************************************************************/
BYTE NrfHwStatus(void)
{
  BYTE Status;

#if 0
  Status = NrfHwReadRegByte(NRF_REG_STATUS);
#else
  // ***** No Operation. Might be used to read the STATUS register
  NRF24L_SET_CS_LOW();
  Status = NrfHwSpiSend(NRF_CMD_NOP);
  NRF24L_SET_CS_HIGH();
#endif  		
  return Status;
}

/*************************************************************************
 * Function Name: NrfHwWriteReg
 * Parameters: Register Address, pointer to Data array, Buffer Size
 *
 * Return: void
 *
 * Description: Write given data to Nrf24L01 register
 *
 *************************************************************************/
void NrfHwWriteReg(BYTE Addr, BYTE *pData, BYTE Size)
{
  BYTE i;

	NRF24L_SET_CS_LOW();
	_nrfLastResult = NrfHwSpiSend(NRF_CMD_WRITE_REG | Addr);
	for(i = 0; i < Size; i++)
	{
		NrfHwSpiSend(*pData++);
	}
	NRF24L_SET_CS_HIGH();
}

/*************************************************************************
 * Function Name: NrfHwWriteRegByte
 * Parameters: Register Address, Data byte
 *
 * Return: none
 *
 * Description: Write given data byte to Nrf24L01 register
 *
 *************************************************************************/
void NrfHwWriteRegByte(BYTE Addr, BYTE Data)
{
	NRF24L_SET_CS_LOW();
	_nrfLastResult = NrfHwSpiSend(NRF_CMD_WRITE_REG | Addr);
  NrfHwSpiSend(Data);
	NRF24L_SET_CS_HIGH();
}

/*************************************************************************
 * Function Name: NrfHwReadReg
 * Parameters: Register Address, pointer to Data array, Buffer Size
 *
 * Return: State of read access
 *
 * Description: Read data from selected Nrf24L01 register
 *
 *************************************************************************/
void NrfHwReadReg(BYTE Addr, BYTE *pData, BYTE Size)
{
  BYTE i;

	NRF24L_SET_CS_LOW();
	_nrfLastResult = NrfHwSpiSend(Addr);
	for(i = 0; i < Size; ++i)
	{
		*pData++ = NrfHwSpiSend(NRF_CMD_NOP);
	}
	NRF24L_SET_CS_HIGH();
}

/*************************************************************************
 * Function Name: NrfHwReadRegByte
 * Parameters: Register Address
 *
 * Return: Data byte
 *
 * Description: Read data byte from selected Nrf24L01 register
 *
 *************************************************************************/
BYTE NrfHwReadRegByte(BYTE Addr)
{
  BYTE Data;

	NRF24L_SET_CS_LOW();
	_nrfLastResult = NrfHwSpiSend(Addr);
	Data = NrfHwSpiSend(NRF_CMD_NOP);
	NRF24L_SET_CS_HIGH();
	return(Data);
}

/*************************************************************************
 * Function Name: NrfHwWriteActRegByte
 * Parameters: Register Address, Data byte
 *
 * Return: None
 * Note: special ACTIVATE SPI command followed by data 0x73.
 *
 * Description: Write data to given Nrf24L01 register
 *
 *************************************************************************/
void NrfHwWriteActRegByte(BYTE Addr, BYTE Data)
{
  _NrfHwSendActComm();
  NrfHwWriteRegByte(Addr, Data);
  _NrfHwSendActComm();
}

/*************************************************************************
 * Function Name: NrfHwReadActRegByte
 * Parameters: Register Address 
 *
 * Return: Data byte
 * Note: special ACTIVATE SPI command followed by data 0x73.
 *
 * Description: Read data from given Nrf24L01 register
 *
 *************************************************************************/
BYTE NrfHwReadActRegByte(BYTE Addr)
{
  BYTE Data;

  _NrfHwSendActComm();
  Data = NrfHwReadRegByte(Addr);
  _NrfHwSendActComm();
  return Data;
}

/*************************************************************************
 * Function Name: NrfHwFlushRx
 * Parameters: None
 *
 * Return: None
 * Note: Status of write access will not be checked here
 *
 * Description: Clear Nrf24L01 Rx FIFO
 *
 *************************************************************************/
void NrfHwFlushRx(void)
{
  NRF24L_SET_CS_LOW();
  NrfHwSpiSend(NRF_CMD_FLUSH_RX);
  NRF24L_SET_CS_HIGH();
}

/*************************************************************************
 * Function Name: NrfHwFlushTx
 * Parameters: None
 *
 * Return: None
 * Note: Status of write access will not be checked here
 *
 * Description: Clear Nrf24L01 Tx FIFO
 *
 *************************************************************************/
void NrfHwFlushTx(void)
{
  NRF24L_SET_CS_LOW();
  NrfHwSpiSend(NRF_CMD_FLUSH_TX);
  NRF24L_SET_CS_HIGH();
}

/*************************************************************************
 * Function Name: NrfHwTransmitPulse
 * Parameters: None
 *
 * Return: None
 *
 * Description: specific target implementation for 10us pulse
 *
 *************************************************************************/
void NrfHwTransmitPulse(void)
{
  int  idx = NRF_TX_PULSE_DLY;

  // ***** A high pulse on CE starts the transmission. 
  // ***** The minimum pulse width on CE is 10 s.
  NRF24L_SET_CE_HIGH();
  while(idx--)
  {
    NRF24L_SET_CE_HIGH();
  }
  NRF24L_SET_CE_LOW();
}

/*************************************************************************
 * Function Name: NrfHwSpiSend
 * Parameters: Data byte
 *
 * Return: Received SPI data byte
 *
 * Description: specific target implementation for SPI access
 *
 *************************************************************************/
#if SAM7_TARGET == NRF_TARGET
BYTE NrfHwSpiSend(BYTE data)
{
  BYTE spib;

  while(!(AT91C_BASE_SPI->SPI_SR & AT91C_SPI_TDRE));    // Wait for the transfer to complete
  AT91C_BASE_SPI->SPI_TDR = data;                       // Send the data

  while(!(AT91C_BASE_SPI->SPI_SR & AT91C_SPI_RDRF));    // Wait until the character can be sent
  spib = (AT91C_BASE_SPI->SPI_RDR);                     // Get the data received
  return spib;
}
#endif

#if AVR_TARGET == NRF_TARGET
BYTE NrfHwSpiSend(BYTE data)
{
  SPDR = data;                    // send Character
  while (!(SPSR & (1 << SPIF)));  // wait until Char is sent
  return SPDR; 
}
#endif
